//          Copyright Jean Pierre Cimalando 2018.
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#include "system_exclusive.h"
#include "utility.h"
#include "app_i18n.h"
#include <fmidi/fmidi.h>
#include <FL/fl_utf8.h>

struct Manufacturer {
    uint8_t length;
    uint8_t code[3];
    const char *name;
};

static const Manufacturer manufacturers[] = {
    {1, {0x01}, "Sequential"},
    {1, {0x02}, "IDP"},
    {1, {0x03}, "Voyetra Turtle Beach, Inc."},
    {1, {0x04}, "Moog Music"},
    {1, {0x05}, "Passport Designs"},
    {1, {0x06}, "Lexicon Inc."},
    {1, {0x07}, "Kurzweil / Young Chang"},
    {1, {0x08}, "Fender"},
    {1, {0x09}, "MIDI9"},
    {1, {0x0A}, "AKG Acoustics"},
    {1, {0x0B}, "Voyce Music"},
    {1, {0x0C}, "WaveFrame (Timeline)"},
    {1, {0x0D}, "ADA Signal Processors, Inc."},
    {1, {0x0E}, "Garfield Electronics"},
    {1, {0x0F}, "Ensoniq"},
    {1, {0x10}, "Oberheim / Gibson Labs"},
    {1, {0x11}, "Apple, Inc."},
    {1, {0x12}, "Grey Matter Response"},
    {1, {0x13}, "Digidesign Inc."},
    {1, {0x14}, "Palmtree Instruments"},
    {1, {0x15}, "JLCooper Electronics"},
    {1, {0x16}, "Lowrey Organ Company"},
    {1, {0x17}, "Adams-Smith"},
    {1, {0x18}, "E-mu"},
    {1, {0x19}, "Harmony Systems"},
    {1, {0x1A}, "ART"},
    {1, {0x1B}, "Baldwin"},
    {1, {0x1C}, "Eventide"},
    {1, {0x1D}, "Inventronics"},
    {1, {0x1E}, "Key Concepts"},
    {1, {0x1F}, "Clarity"},
    {1, {0x20}, "Passac"},
    {1, {0x21}, "Proel Labs (SIEL)"},
    {1, {0x22}, "Synthaxe (UK)"},
    {1, {0x23}, "Stepp"},
    {1, {0x24}, "Hohner"},
    {1, {0x25}, "Twister"},
    {1, {0x26}, "Ketron s.r.l."},
    {1, {0x27}, "Jellinghaus MS"},
    {1, {0x28}, "Southworth Music Systems"},
    {1, {0x29}, "PPG (Germany)"},
    {1, {0x2A}, "JEN"},
    {1, {0x2B}, "Solid State Logic Organ Systems"},
    {1, {0x2C}, "Audio Veritrieb-P. Struven"},
    {1, {0x2D}, "Neve"},
    {1, {0x2E}, "Soundtracs Ltd."},
    {1, {0x2F}, "Elka"},
    {1, {0x30}, "Dynacord"},
    {1, {0x31}, "Viscount International Spa (Intercontinental Electronics)"},
    {1, {0x32}, "Drawmer"},
    {1, {0x33}, "Clavia Digital Instruments"},
    {1, {0x34}, "Audio Architecture"},
    {1, {0x35}, "Generalmusic Corp SpA"},
    {1, {0x36}, "Cheetah Marketing"},
    {1, {0x37}, "C.T.M."},
    {1, {0x38}, "Simmons UK"},
    {1, {0x39}, "Soundcraft Electronics"},
    {1, {0x3A}, "Steinberg Media Technologies AG"},
    {1, {0x3B}, "Wersi Gmbh"},
    {1, {0x3C}, "AVAB Niethammer AB"},
    {1, {0x3D}, "Digigram"},
    {1, {0x3E}, "Waldorf Electronics GmbH 3FH Quasimidi"},
    {3, {0x00, 0x00, 0x01}, "Time/Warner Interactive"},
    {3, {0x00, 0x00, 0x02}, "Advanced Gravis Comp. Tech Ltd."},
    {3, {0x00, 0x00, 0x03}, "Media Vision"},
    {3, {0x00, 0x00, 0x04}, "Dornes Research Group"},
    {3, {0x00, 0x00, 0x05}, "K-Muse"},
    {3, {0x00, 0x00, 0x06}, "Stypher"},
    {3, {0x00, 0x00, 0x07}, "Digital Music Corp."},
    {3, {0x00, 0x00, 0x08}, "IOTA Systems"},
    {3, {0x00, 0x00, 0x09}, "New England Digital"},
    {3, {0x00, 0x00, 0x0A}, "Artisyn"},
    {3, {0x00, 0x00, 0x0B}, "IVL Technologies Ltd."},
    {3, {0x00, 0x00, 0x0C}, "Southern Music Systems"},
    {3, {0x00, 0x00, 0x0D}, "Lake Butler Sound Company"},
    {3, {0x00, 0x00, 0x0E}, "Alesis Studio Electronics"},
    {3, {0x00, 0x00, 0x0F}, "Sound Creation"},
    {3, {0x00, 0x00, 0x01}, "Time/Warner Interactive"},
    {3, {0x00, 0x00, 0x10}, "DOD Electronics Corp."},
    {3, {0x00, 0x00, 0x11}, "Studer-Editech"},
    {3, {0x00, 0x00, 0x12}, "Sonus"},
    {3, {0x00, 0x00, 0x13}, "Temporal Acuity Products"},
    {3, {0x00, 0x00, 0x14}, "Perfect Fretworks"},
    {3, {0x00, 0x00, 0x15}, "KAT Inc."},
    {3, {0x00, 0x00, 0x16}, "Opcode Systems"},
    {3, {0x00, 0x00, 0x17}, "Rane Corporation"},
    {3, {0x00, 0x00, 0x18}, "Anadi Electronique"},
    {3, {0x00, 0x00, 0x19}, "KMX"},
    {3, {0x00, 0x00, 0x1A}, "Allen & Heath Brenell"},
    {3, {0x00, 0x00, 0x1B}, "Peavey Electronics"},
    {3, {0x00, 0x00, 0x1C}, "360 Systems"},
    {3, {0x00, 0x00, 0x1D}, "Spectrum Design and Development"},
    {3, {0x00, 0x00, 0x1E}, "Marquis Music"},
    {3, {0x00, 0x00, 0x1F}, "Zeta Systems"},
    {3, {0x00, 0x00, 0x20}, "Axxes (Brian Parsonett)"},
    {3, {0x00, 0x00, 0x21}, "Orban"},
    {3, {0x00, 0x00, 0x22}, "Indian Valley Mfg."},
    {3, {0x00, 0x00, 0x23}, "Triton"},
    {3, {0x00, 0x00, 0x24}, "KTI"},
    {3, {0x00, 0x00, 0x25}, "Breakaway Technologies"},
    {3, {0x00, 0x00, 0x26}, "Leprecon / CAE Inc."},
    {3, {0x00, 0x00, 0x27}, "Harrison Systems Inc."},
    {3, {0x00, 0x00, 0x28}, "Future Lab/Mark Kuo"},
    {3, {0x00, 0x00, 0x29}, "Rocktron Corporation"},
    {3, {0x00, 0x00, 0x2A}, "PianoDisc"},
    {3, {0x00, 0x00, 0x2B}, "Cannon Research Group"},
    {3, {0x00, 0x00, 0x2C}, "Reserved"},
    {3, {0x00, 0x00, 0x2D}, "Rodgers Instrument LLC"},
    {3, {0x00, 0x00, 0x2E}, "Blue Sky Logic"},
    {3, {0x00, 0x00, 0x2F}, "Encore Electronics"},
    {3, {0x00, 0x00, 0x30}, "Uptown"},
    {3, {0x00, 0x00, 0x31}, "Voce"},
    {3, {0x00, 0x00, 0x32}, "CTI Audio, Inc. (Musically Intel. Devs.)"},
    {3, {0x00, 0x00, 0x33}, "S3 Incorporated"},
    {3, {0x00, 0x00, 0x34}, "Broderbund / Red Orb"},
    {3, {0x00, 0x00, 0x35}, "Allen Organ Co."},
    {3, {0x00, 0x00, 0x36}, "Reserved"},
    {3, {0x00, 0x00, 0x37}, "Music Quest"},
    {3, {0x00, 0x00, 0x38}, "Aphex"},
    {3, {0x00, 0x00, 0x39}, "Gallien Krueger"},
    {3, {0x00, 0x00, 0x3A}, "IBM"},
    {3, {0x00, 0x00, 0x3B}, "Mark Of The Unicorn"},
    {3, {0x00, 0x00, 0x3C}, "Hotz Corporation"},
    {3, {0x00, 0x00, 0x3D}, "ETA Lighting"},
    {3, {0x00, 0x00, 0x3E}, "NSI Corporation"},
    {3, {0x00, 0x00, 0x3F}, "Ad Lib, Inc."},
    {3, {0x00, 0x00, 0x40}, "Richmond Sound Design"},
    {3, {0x00, 0x00, 0x41}, "Microsoft"},
    {3, {0x00, 0x00, 0x42}, "Mindscape (Software Toolworks)"},
    {3, {0x00, 0x00, 0x43}, "Russ Jones Marketing / Niche"},
    {3, {0x00, 0x00, 0x44}, "Intone"},
    {3, {0x00, 0x00, 0x45}, "Advanced Remote Technologies"},
    {3, {0x00, 0x00, 0x46}, "White Instruments"},
    {3, {0x00, 0x00, 0x47}, "GT Electronics/Groove Tubes"},
    {3, {0x00, 0x00, 0x48}, "Pacific Research & Engineering"},
    {3, {0x00, 0x00, 0x49}, "Timeline Vista, Inc."},
    {3, {0x00, 0x00, 0x4A}, "Mesa Boogie Ltd."},
    {3, {0x00, 0x00, 0x4B}, "FSLI"},
    {3, {0x00, 0x00, 0x4C}, "Sequoia Development Group"},
    {3, {0x00, 0x00, 0x4D}, "Studio Electronics"},
    {3, {0x00, 0x00, 0x4E}, "Euphonix, Inc"},
    {3, {0x00, 0x00, 0x4F}, "InterMIDI, Inc."},
    {3, {0x00, 0x00, 0x50}, "MIDI Solutions Inc."},
    {3, {0x00, 0x00, 0x51}, "3DO Company"},
    {3, {0x00, 0x00, 0x52}, "Lightwave Research / High End Systems"},
    {3, {0x00, 0x00, 0x53}, "Micro-W Corporation"},
    {3, {0x00, 0x00, 0x54}, "Spectral Synthesis, Inc."},
    {3, {0x00, 0x00, 0x55}, "Lone Wolf"},
    {3, {0x00, 0x00, 0x56}, "Studio Technologies Inc."},
    {3, {0x00, 0x00, 0x57}, "Peterson Electro-Musical Product, Inc."},
    {3, {0x00, 0x00, 0x58}, "Atari Corporation"},
    {3, {0x00, 0x00, 0x59}, "Marion Systems Corporation"},
    {3, {0x00, 0x00, 0x5A}, "Design Event"},
    {3, {0x00, 0x00, 0x5B}, "Winjammer Software Ltd."},
    {3, {0x00, 0x00, 0x5C}, "AT&T Bell Laboratories"},
    {3, {0x00, 0x00, 0x5D}, "Reserved"},
    {3, {0x00, 0x00, 0x5E}, "Symetrix"},
    {3, {0x00, 0x00, 0x5F}, "MIDI the World"},
    {3, {0x00, 0x00, 0x60}, "Spatializer"},
    {3, {0x00, 0x00, 0x61}, "Micros 'N MIDI"},
    {3, {0x00, 0x00, 0x62}, "Accordians International"},
    {3, {0x00, 0x00, 0x63}, "EuPhonics (now 3Com)"},
    {3, {0x00, 0x00, 0x64}, "Musonix"},
    {3, {0x00, 0x00, 0x65}, "Turtle Beach Systems (Voyetra)"},
    {3, {0x00, 0x00, 0x66}, "Loud Technologies / Mackie"},
    {3, {0x00, 0x00, 0x67}, "Compuserve"},
    {3, {0x00, 0x00, 0x68}, "BEC Technologies"},
    {3, {0x00, 0x00, 0x69}, "QRS Music Inc"},
    {3, {0x00, 0x00, 0x6A}, "P.G. Music"},
    {3, {0x00, 0x00, 0x6B}, "Sierra Semiconductor"},
    {3, {0x00, 0x00, 0x6C}, "EpiGraf"},
    {3, {0x00, 0x00, 0x6D}, "Electronics Diversified Inc"},
    {3, {0x00, 0x00, 0x6E}, "Tune 1000"},
    {3, {0x00, 0x00, 0x6F}, "Advanced Micro Devices"},
    {3, {0x00, 0x00, 0x70}, "Mediamation"},
    {3, {0x00, 0x00, 0x71}, "Sabine Musical Mfg. Co. Inc."},
    {3, {0x00, 0x00, 0x72}, "Woog Labs"},
    {3, {0x00, 0x00, 0x73}, "Micropolis Corp"},
    {3, {0x00, 0x00, 0x74}, "Ta Horng Musical Instrument"},
    {3, {0x00, 0x00, 0x75}, "e-Tek Labs (Forte Tech)"},
    {3, {0x00, 0x00, 0x76}, "Electro-Voice"},
    {3, {0x00, 0x00, 0x77}, "Midisoft Corporation"},
    {3, {0x00, 0x00, 0x78}, "QSound Labs"},
    {3, {0x00, 0x00, 0x79}, "Westrex"},
    {3, {0x00, 0x00, 0x7A}, "Nvidia"},
    {3, {0x00, 0x00, 0x7B}, "ESS Technology"},
    {3, {0x00, 0x00, 0x7C}, "Media Trix Peripherals"},
    {3, {0x00, 0x00, 0x7D}, "Brooktree Corp"},
    {3, {0x00, 0x00, 0x7E}, "Otari Corp"},
    {3, {0x00, 0x00, 0x7F}, "Key Electronics, Inc."},
    {3, {0x00, 0x01, 0x00}, "Shure Incorporated"},
    {3, {0x00, 0x01, 0x01}, "AuraSound"},
    {3, {0x00, 0x01, 0x02}, "Crystal Semiconductor"},
    {3, {0x00, 0x01, 0x03}, "Conexant (Rockwell)"},
    {3, {0x00, 0x01, 0x04}, "Silicon Graphics"},
    {3, {0x00, 0x01, 0x05}, "M-Audio (Midiman)"},
    {3, {0x00, 0x01, 0x06}, "PreSonus"},
    {3, {0x00, 0x01, 0x08}, "Topaz Enterprises"},
    {3, {0x00, 0x01, 0x09}, "Cast Lighting"},
    {3, {0x00, 0x01, 0x0A}, "Microsoft"},
    {3, {0x00, 0x01, 0x0B}, "Sonic Foundry"},
    {3, {0x00, 0x01, 0x0C}, "Line 6 (Fast Forward) (Yamaha)"},
    {3, {0x00, 0x01, 0x0D}, "Beatnik Inc"},
    {3, {0x00, 0x01, 0x0E}, "Van Koevering Company"},
    {3, {0x00, 0x01, 0x0F}, "Altech Systems"},
    {3, {0x00, 0x01, 0x10}, "S & S Research"},
    {3, {0x00, 0x01, 0x11}, "VLSI Technology"},
    {3, {0x00, 0x01, 0x12}, "Chromatic Research"},
    {3, {0x00, 0x01, 0x13}, "Sapphire"},
    {3, {0x00, 0x01, 0x14}, "IDRC"},
    {3, {0x00, 0x01, 0x15}, "Justonic Tuning"},
    {3, {0x00, 0x01, 0x16}, "TorComp Research Inc."},
    {3, {0x00, 0x01, 0x17}, "Newtek Inc."},
    {3, {0x00, 0x01, 0x18}, "Sound Sculpture"},
    {3, {0x00, 0x01, 0x19}, "Walker Technical"},
    {3, {0x00, 0x01, 0x1A}, "Digital Harmony (PAVO)"},
    {3, {0x00, 0x01, 0x1B}, "InVision Interactive"},
    {3, {0x00, 0x01, 0x1C}, "T-Square Design"},
    {3, {0x00, 0x01, 0x1D}, "Nemesys Music Technology"},
    {3, {0x00, 0x01, 0x1E}, "DBX Professional (Harman Intl)"},
    {3, {0x00, 0x01, 0x1F}, "Syndyne Corporation"},
    {3, {0x00, 0x01, 0x20}, "Bitheadz"},
    {3, {0x00, 0x01, 0x21}, "Cakewalk Music Software"},
    {3, {0x00, 0x01, 0x22}, "Analog Devices"},
    {3, {0x00, 0x01, 0x23}, "National Semiconductor"},
    {3, {0x00, 0x01, 0x24}, "Boom Theory / Adinolfi Alternative Percussion"},
    {3, {0x00, 0x01, 0x25}, "Virtual DSP Corporation"},
    {3, {0x00, 0x01, 0x26}, "Antares Systems"},
    {3, {0x00, 0x01, 0x27}, "Angel Software"},
    {3, {0x00, 0x01, 0x28}, "St Louis Music"},
    {3, {0x00, 0x01, 0x29}, "Passport Music Software LLC (Gvox)"},
    {3, {0x00, 0x01, 0x2A}, "Ashley Audio Inc."},
    {3, {0x00, 0x01, 0x2B}, "Vari-Lite Inc."},
    {3, {0x00, 0x01, 0x2C}, "Summit Audio Inc."},
    {3, {0x00, 0x01, 0x2D}, "Aureal Semiconductor Inc."},
    {3, {0x00, 0x01, 0x2E}, "SeaSound LLC"},
    {3, {0x00, 0x01, 0x2F}, "U.S. Robotics"},
    {3, {0x00, 0x01, 0x30}, "Aurisis Research"},
    {3, {0x00, 0x01, 0x31}, "Nearfield Research"},
    {3, {0x00, 0x01, 0x32}, "FM7 Inc"},
    {3, {0x00, 0x01, 0x33}, "Swivel Systems"},
    {3, {0x00, 0x01, 0x34}, "Hyperactive Audio Systems"},
    {3, {0x00, 0x01, 0x35}, "MidiLite (Castle Studios Productions)"},
    {3, {0x00, 0x01, 0x36}, "Radikal Technologies"},
    {3, {0x00, 0x01, 0x37}, "Roger Linn Design"},
    {3, {0x00, 0x01, 0x38}, "TC-Helicon Vocal Technologies"},
    {3, {0x00, 0x01, 0x39}, "Event Electronics"},
    {3, {0x00, 0x01, 0x3A}, "Sonic Network Inc"},
    {3, {0x00, 0x01, 0x3B}, "Realtime Music Solutions"},
    {3, {0x00, 0x01, 0x3C}, "Apogee Digital"},
    {3, {0x00, 0x01, 0x3D}, "Classical Organs, Inc."},
    {3, {0x00, 0x01, 0x3E}, "Microtools Inc."},
    {3, {0x00, 0x01, 0x3F}, "Numark Industries"},
    {3, {0x00, 0x01, 0x40}, "Frontier Design Group, LLC"},
    {3, {0x00, 0x01, 0x41}, "Recordare LLC"},
    {3, {0x00, 0x01, 0x42}, "Starr Labs"},
    {3, {0x00, 0x01, 0x43}, "Voyager Sound Inc."},
    {3, {0x00, 0x01, 0x44}, "Manifold Labs"},
    {3, {0x00, 0x01, 0x45}, "Aviom Inc."},
    {3, {0x00, 0x01, 0x46}, "Mixmeister Technology"},
    {3, {0x00, 0x01, 0x47}, "Notation Software"},
    {3, {0x00, 0x01, 0x48}, "Mercurial Communications"},
    {3, {0x00, 0x01, 0x49}, "Wave Arts"},
    {3, {0x00, 0x01, 0x4A}, "Logic Sequencing Devices"},
    {3, {0x00, 0x01, 0x4B}, "Axess Electronics"},
    {3, {0x00, 0x01, 0x4C}, "Muse Research"},
    {3, {0x00, 0x01, 0x4D}, "Open Labs"},
    {3, {0x00, 0x01, 0x4E}, "Guillemot Corp"},
    {3, {0x00, 0x01, 0x4F}, "Samson Technologies"},
    {3, {0x00, 0x01, 0x50}, "Electronic Theatre Controls"},
    {3, {0x00, 0x01, 0x51}, "Blackberry (RIM)"},
    {3, {0x00, 0x01, 0x52}, "Mobileer"},
    {3, {0x00, 0x01, 0x53}, "Synthogy"},
    {3, {0x00, 0x01, 0x54}, "Lynx Studio Technology Inc."},
    {3, {0x00, 0x01, 0x55}, "Damage Control Engineering LLC"},
    {3, {0x00, 0x01, 0x56}, "Yost Engineering, Inc."},
    {3, {0x00, 0x01, 0x57}, "Brooks & Forsman Designs LLC / DrumLite"},
    {3, {0x00, 0x01, 0x58}, "Infinite Response"},
    {3, {0x00, 0x01, 0x59}, "Garritan Corp"},
    {3, {0x00, 0x01, 0x5A}, "Plogue Art et Technologie, Inc"},
    {3, {0x00, 0x01, 0x5B}, "RJM Music Technology"},
    {3, {0x00, 0x01, 0x5C}, "Custom Solutions Software"},
    {3, {0x00, 0x01, 0x5D}, "Sonarcana LLC / Highly Liquid"},
    {3, {0x00, 0x01, 0x5E}, "Centrance"},
    {3, {0x00, 0x01, 0x5F}, "Kesumo LLC"},
    {3, {0x00, 0x01, 0x60}, "Stanton  (Gibson Brands)"},
    {3, {0x00, 0x01, 0x61}, "Livid Instruments"},
    {3, {0x00, 0x01, 0x62}, "First Act / 745 Media"},
    {3, {0x00, 0x01, 0x63}, "Pygraphics, Inc."},
    {3, {0x00, 0x01, 0x64}, "Panadigm Innovations Ltd"},
    {3, {0x00, 0x01, 0x65}, "Avedis Zildjian Co"},
    {3, {0x00, 0x01, 0x66}, "Auvital Music Corp"},
    {3, {0x00, 0x01, 0x67}, "You Rock Guitar (was: Inspired Instruments)"},
    {3, {0x00, 0x01, 0x68}, "Chris Grigg Designs"},
    {3, {0x00, 0x01, 0x69}, "Slate Digital LLC"},
    {3, {0x00, 0x01, 0x6A}, "Mixware"},
    {3, {0x00, 0x01, 0x6B}, "Social Entropy"},
    {3, {0x00, 0x01, 0x6C}, "Source Audio LLC"},
    {3, {0x00, 0x01, 0x6D}, "Ernie Ball / Music Man"},
    {3, {0x00, 0x01, 0x6E}, "Fishman"},
    {3, {0x00, 0x01, 0x6F}, "Custom Audio Electronics"},
    {3, {0x00, 0x01, 0x70}, "American Audio/DJ"},
    {3, {0x00, 0x01, 0x71}, "Mega Control Systems"},
    {3, {0x00, 0x01, 0x72}, "Kilpatrick Audio"},
    {3, {0x00, 0x01, 0x73}, "iConnectivity"},
    {3, {0x00, 0x01, 0x74}, "Fractal Audio"},
    {3, {0x00, 0x01, 0x75}, "NetLogic Microsystems"},
    {3, {0x00, 0x01, 0x76}, "Music Computing"},
    {3, {0x00, 0x01, 0x77}, "Nektar Technology Inc"},
    {3, {0x00, 0x01, 0x78}, "Zenph Sound Innovations"},
    {3, {0x00, 0x01, 0x79}, "DJTechTools.com"},
    {3, {0x00, 0x01, 0x7A}, "Rezonance Labs"},
    {3, {0x00, 0x01, 0x7B}, "Decibel Eleven"},
    {3, {0x00, 0x01, 0x7C}, "CNMAT"},
    {3, {0x00, 0x01, 0x7D}, "Media Overkill"},
    {3, {0x00, 0x01, 0x7E}, "Confusion Studios"},
    {3, {0x00, 0x01, 0x7F}, "moForte Inc"},
    {3, {0x00, 0x02, 0x00}, "Miselu Inc"},
    {3, {0x00, 0x02, 0x01}, "Amelia's Compass LLC"},
    {3, {0x00, 0x02, 0x02}, "Zivix LLC"},
    {3, {0x00, 0x02, 0x03}, "Artiphon"},
    {3, {0x00, 0x02, 0x04}, "Synclavier Digital"},
    {3, {0x00, 0x02, 0x05}, "Light & Sound Control Devices LLC"},
    {3, {0x00, 0x02, 0x06}, "Retronyms Inc"},
    {3, {0x00, 0x02, 0x07}, "JS Technologies"},
    {3, {0x00, 0x02, 0x08}, "Quicco Sound"},
    {3, {0x00, 0x02, 0x09}, "A-Designs Audio"},
    {3, {0x00, 0x02, 0x0A}, "McCarthy Music Corp"},
    {3, {0x00, 0x02, 0x0B}, "Denon DJ"},
    {3, {0x00, 0x02, 0x0C}, "Keith Robert Murray"},
    {3, {0x00, 0x02, 0x0D}, "Google"},
    {3, {0x00, 0x02, 0x0E}, "ISP Technologies"},
    {3, {0x00, 0x02, 0x0F}, "Abstrakt Instruments LLC"},
    {3, {0x00, 0x02, 0x10}, "Meris LLC"},
    {3, {0x00, 0x02, 0x11}, "Sensorpoint LLC"},
    {3, {0x00, 0x02, 0x12}, "Hi-Z Labs"},
    {3, {0x00, 0x02, 0x13}, "Imitone"},
    {3, {0x00, 0x02, 0x14}, "Intellijel Designs Inc."},
    {3, {0x00, 0x02, 0x15}, "Dasz Instruments Inc."},
    {3, {0x00, 0x02, 0x16}, "Remidi"},
    {3, {0x00, 0x02, 0x17}, "Disaster Area Designs LLC"},
    {3, {0x00, 0x02, 0x18}, "Universal Audio"},
    {3, {0x00, 0x02, 0x19}, "Carter Duncan Corp"},
    {3, {0x00, 0x02, 0x1A}, "Essential Technology"},
    {3, {0x00, 0x02, 0x1B}, "Cantux Research LLC"},
    {3, {0x00, 0x02, 0x1C}, "Hummel Technologies"},
    {3, {0x00, 0x02, 0x1D}, "Sensel Inc"},
    {3, {0x00, 0x02, 0x1E}, "DBML Group"},
    {3, {0x00, 0x02, 0x1F}, "Madrona Labs"},
    {3, {0x00, 0x02, 0x20}, "Mesa Boogie"},
    {3, {0x00, 0x02, 0x21}, "Effigy Labs"},
    {3, {0x00, 0x02, 0x22}, "MK2 Image Ltd"},
    {3, {0x00, 0x02, 0x23}, "Red Panda LLC"},
    {3, {0x00, 0x02, 0x24}, "OnSong LLC"},
    {3, {0x00, 0x02, 0x25}, "Jamboxx Inc."},
    {3, {0x00, 0x02, 0x26}, "Electro-Harmonix"},
    {3, {0x00, 0x02, 0x27}, "RnD64 Inc"},
    {3, {0x00, 0x02, 0x28}, "Neunaber Technology LLC"},
    {3, {0x00, 0x20, 0x00}, "Dream SAS"},
    {3, {0x00, 0x20, 0x01}, "Strand Lighting"},
    {3, {0x00, 0x20, 0x02}, "Amek Div of Harman Industries"},
    {3, {0x00, 0x20, 0x03}, "Casa Di Risparmio Di Loreto"},
    {3, {0x00, 0x20, 0x04}, "Böhm electronic GmbH 00H 20H 05H Syntec Digital Audio"},
    {3, {0x00, 0x20, 0x06}, "Trident Audio Developments"},
    {3, {0x00, 0x20, 0x07}, "Real World Studio"},
    {3, {0x00, 0x20, 0x08}, "Evolution Synthesis, Ltd"},
    {3, {0x00, 0x20, 0x09}, "Yes Technology"},
    {3, {0x00, 0x20, 0x0A}, "Audiomatica"},
    {3, {0x00, 0x20, 0x0B}, "Bontempi SpA (Sigma)"},
    {3, {0x00, 0x20, 0x0C}, "F.B.T. Elettronica SpA"},
    {3, {0x00, 0x20, 0x0D}, "MidiTemp GmbH 00H 20H 0EH LA Audio (Larking Audio)"},
    {3, {0x00, 0x20, 0x0F}, "Zero 88 Lighting Limited"},
    {3, {0x00, 0x20, 0x10}, "Micon Audio Electronics GmbH 00H 20H 11H Forefront Technology"},
    {3, {0x00, 0x20, 0x12}, "Studio Audio and Video Ltd."},
    {3, {0x00, 0x20, 0x13}, "Kenton Electronics"},
    {3, {0x00, 0x20, 0x14}, "Celco/ Electrosonic"},
    {3, {0x00, 0x20, 0x15}, "ADB"},
    {3, {0x00, 0x20, 0x16}, "Marshall Products Limited"},
    {3, {0x00, 0x20, 0x17}, "DDA"},
    {3, {0x00, 0x20, 0x18}, "BSS Audio Ltd."},
    {3, {0x00, 0x20, 0x19}, "MA Lighting Technology"},
    {3, {0x00, 0x20, 0x1A}, "Fatar SRL c/o Music Industries"},
    {3, {0x00, 0x20, 0x1B}, "QSC Audio Products Inc."},
    {3, {0x00, 0x20, 0x1C}, "Artisan Clasic Organ Inc."},
    {3, {0x00, 0x20, 0x1D}, "Orla Spa"},
    {3, {0x00, 0x20, 0x1E}, "Pinnacle Audio (Klark Teknik PLC)"},
    {3, {0x00, 0x20, 0x1F}, "TC Electronics"},
    {3, {0x00, 0x20, 0x20}, "Doepfer Musikelektronik GmbH 00H 20H 21H Creative ATC / E-mu"},
    {3, {0x00, 0x20, 0x22}, "Seyddo/Minami"},
    {3, {0x00, 0x20, 0x23}, "LG Electronics (Goldstar)"},
    {3, {0x00, 0x20, 0x24}, "Midisoft sas di M.Cima & C"},
    {3, {0x00, 0x20, 0x25}, "Samick Musical Inst. Co. Ltd."},
    {3, {0x00, 0x20, 0x26}, "Penny and Giles (Bowthorpe PLC)"},
    {3, {0x00, 0x20, 0x27}, "Acorn Computer"},
    {3, {0x00, 0x20, 0x28}, "LSC Electronics Pty. Ltd."},
    {3, {0x00, 0x20, 0x29}, "Focusrite/Novation"},
    {3, {0x00, 0x20, 0x2A}, "Samkyung Mechatronics"},
    {3, {0x00, 0x20, 0x2B}, "Medeli Electronics Co."},
    {3, {0x00, 0x20, 0x2C}, "Charlie Lab SRL"},
    {3, {0x00, 0x20, 0x2D}, "Blue Chip Music Technology"},
    {3, {0x00, 0x20, 0x2E}, "BEE OH Corp"},
    {3, {0x00, 0x20, 0x2F}, "LG Semicon America"},
    {3, {0x00, 0x20, 0x30}, "TESI"},
    {3, {0x00, 0x20, 0x31}, "EMAGIC"},
    {3, {0x00, 0x20, 0x32}, "Behringer GmbH 00H 20H 33H Access Music Electronics"},
    {3, {0x00, 0x20, 0x34}, "Synoptic"},
    {3, {0x00, 0x20, 0x35}, "Hanmesoft"},
    {3, {0x00, 0x20, 0x36}, "Terratec Electronic GmbH 00H 20H 37H Proel SpA"},
    {3, {0x00, 0x20, 0x38}, "IBK MIDI"},
    {3, {0x00, 0x20, 0x39}, "IRCAM"},
    {3, {0x00, 0x20, 0x3A}, "Propellerhead Software"},
    {3, {0x00, 0x20, 0x3B}, "Red Sound Systems Ltd"},
    {3, {0x00, 0x20, 0x3C}, "Elektron ESI AB"},
    {3, {0x00, 0x20, 0x3D}, "Sintefex Audio"},
    {3, {0x00, 0x20, 0x3E}, "MAM (Music and More)"},
    {3, {0x00, 0x20, 0x3F}, "Amsaro GmbH 00H 20H 40H CDS Advanced Technology BV (Lanbox)"},
    {3, {0x00, 0x20, 0x41}, "Mode Machines (Touched By Sound GmbH)"},
    {3, {0x00, 0x20, 0x42}, "DSP Arts"},
    {3, {0x00, 0x20, 0x43}, "Phil Rees Music Tech"},
    {3, {0x00, 0x20, 0x44}, "Stamer Musikanlagen GmbH suspended"},
    {3, {0x00, 0x20, 0x45}, "Musical Muntaner S.A. dba Soundart"},
    {3, {0x00, 0x20, 0x46}, "C-Mexx Software"},
    {3, {0x00, 0x20, 0x47}, "Klavis Technologies"},
    {3, {0x00, 0x20, 0x48}, "Noteheads AB"},
    {3, {0x00, 0x20, 0x49}, "Algorithmix"},
    {3, {0x00, 0x20, 0x4A}, "Skrydstrup R&D"},
    {3, {0x00, 0x20, 0x4B}, "Professional Audio Company"},
    {3, {0x00, 0x20, 0x4C}, "NewWave Labs (MadWaves)"},
    {3, {0x00, 0x20, 0x4D}, "Vermona"},
    {3, {0x00, 0x20, 0x4E}, "Nokia"},
    {3, {0x00, 0x20, 0x4F}, "Wave Idea"},
    {3, {0x00, 0x20, 0x50}, "Hartmann GmbH 00H 20H 51H Lion's Tracs"},
    {3, {0x00, 0x20, 0x52}, "Analogue Systems"},
    {3, {0x00, 0x20, 0x53}, "Focal-JMlab"},
    {3, {0x00, 0x20, 0x54}, "Ringway Electronics (Chang-Zhou) Co Ltd"},
    {3, {0x00, 0x20, 0x55}, "Faith Technologies (Digiplug)"},
    {3, {0x00, 0x20, 0x56}, "Showworks"},
    {3, {0x00, 0x20, 0x57}, "Manikin Electronic"},
    {3, {0x00, 0x20, 0x58}, "1 Come Tech"},
    {3, {0x00, 0x20, 0x59}, "Phonic Corp"},
    {3, {0x00, 0x20, 0x5A}, "Dolby Australia (Lake)"},
    {3, {0x00, 0x20, 0x5B}, "Silansys Technologies"},
    {3, {0x00, 0x20, 0x5C}, "Winbond Electronics"},
    {3, {0x00, 0x20, 0x5D}, "Cinetix Medien und Interface GmbH 00H 20H 5EH A&G Soluzioni Digitali"},
    {3, {0x00, 0x20, 0x5F}, "Sequentix GmbH 00H 20H 60H Oram Pro Audio"},
    {3, {0x00, 0x20, 0x61}, "Be4 Ltd"},
    {3, {0x00, 0x20, 0x62}, "Infection Music"},
    {3, {0x00, 0x20, 0x63}, "Central Music Co. (CME)"},
    {3, {0x00, 0x20, 0x64}, "genoQs Machines GmbH suspended"},
    {3, {0x00, 0x20, 0x65}, "Medialon"},
    {3, {0x00, 0x20, 0x66}, "Waves Audio Ltd"},
    {3, {0x00, 0x20, 0x67}, "Jerash Labs"},
    {3, {0x00, 0x20, 0x68}, "Da Fact"},
    {3, {0x00, 0x20, 0x69}, "Elby Designs"},
    {3, {0x00, 0x20, 0x6A}, "Spectral Audio"},
    {3, {0x00, 0x20, 0x6B}, "Arturia"},
    {3, {0x00, 0x20, 0x6C}, "Vixid"},
    {3, {0x00, 0x20, 0x6D}, "C-Thru Music"},
    {3, {0x00, 0x20, 0x6E}, "Ya Horng Electronic Co LTD"},
    {3, {0x00, 0x20, 0x6F}, "SM Pro Audio"},
    {3, {0x00, 0x20, 0x70}, "OTO Machines"},
    {3, {0x00, 0x20, 0x71}, "ELZAB S.A. (G LAB)"},
    {3, {0x00, 0x20, 0x72}, "Blackstar Amplification Ltd"},
    {3, {0x00, 0x20, 0x73}, "M3i Technologies GmbH 00H 20H 74H Gemalto (from Xiring)"},
    {3, {0x00, 0x20, 0x75}, "Prostage SL"},
    {3, {0x00, 0x20, 0x76}, "Teenage Engineering"},
    {3, {0x00, 0x20, 0x77}, "Tobias Erichsen Consulting"},
    {3, {0x00, 0x20, 0x78}, "Nixer Ltd"},
    {3, {0x00, 0x20, 0x79}, "Hanpin Electron Co Ltd"},
    {3, {0x00, 0x20, 0x7A}, "\"MIDI-hardware\" R.Sowa"},
    {3, {0x00, 0x20, 0x7B}, "Beyond Music Industrial Ltd"},
    {3, {0x00, 0x20, 0x7C}, "Kiss Box B.V."},
    {3, {0x00, 0x20, 0x7D}, "Misa Digital Technologies Ltd"},
    {3, {0x00, 0x20, 0x7E}, "AI Musics Technology Inc"},
    {3, {0x00, 0x20, 0x7F}, "Serato Inc LP"},
    {3, {0x00, 0x21, 0x00}, "Limex"},
    {3, {0x00, 0x21, 0x01}, "Kyodday (Tokai)"},
    {3, {0x00, 0x21, 0x02}, "Mutable Instruments"},
    {3, {0x00, 0x21, 0x03}, "PreSonus Software Ltd"},
    {3, {0x00, 0x21, 0x04}, "Ingenico (was Xiring)"},
    {3, {0x00, 0x21, 0x05}, "Fairlight Instruments Pty Ltd"},
    {3, {0x00, 0x21, 0x06}, "Musicom Lab"},
    {3, {0x00, 0x21, 0x07}, "Modal Electronics (Modulus/VacoLoco)"},
    {3, {0x00, 0x21, 0x08}, "RWA (Hong Kong) Limited"},
    {3, {0x00, 0x21, 0x09}, "Native Instruments"},
    {3, {0x00, 0x21, 0x0A}, "Naonext"},
    {3, {0x00, 0x21, 0x0B}, "MFB"},
    {3, {0x00, 0x21, 0x0C}, "Teknel Research"},
    {3, {0x00, 0x21, 0x0D}, "Ploytec GmbH 00H 21H 0EH Surfin Kangaroo Studio"},
    {3, {0x00, 0x21, 0x0F}, "Philips Electronics HK Ltd"},
    {3, {0x00, 0x21, 0x10}, "ROLI Ltd"},
    {3, {0x00, 0x21, 0x11}, "Panda-Audio Ltd"},
    {3, {0x00, 0x21, 0x12}, "BauM Software"},
    {3, {0x00, 0x21, 0x13}, "Machinewerks Ltd."},
    {3, {0x00, 0x21, 0x14}, "Xiamen Elane Electronics"},
    {3, {0x00, 0x21, 0x15}, "Marshall Amplification PLC"},
    {3, {0x00, 0x21, 0x16}, "Kiwitechnics Ltd"},
    {3, {0x00, 0x21, 0x17}, "Rob Papen"},
    {3, {0x00, 0x21, 0x18}, "Spicetone OU"},
    {3, {0x00, 0x21, 0x19}, "V3Sound"},
    {3, {0x00, 0x21, 0x1A}, "IK Multimedia"},
    {3, {0x00, 0x21, 0x1B}, "Novalia Ltd"},
    {3, {0x00, 0x21, 0x1C}, "Modor Music"},
    {3, {0x00, 0x21, 0x1D}, "Ableton"},
    {3, {0x00, 0x21, 0x1E}, "Dtronics"},
    {3, {0x00, 0x21, 0x1F}, "ZAQ Audio"},
    {3, {0x00, 0x21, 0x20}, "Muabaobao Education Technology Co Ltd"},
    {3, {0x00, 0x21, 0x21}, "Flux Effects"},
    {3, {0x00, 0x21, 0x22}, "Audiothingies (MCDA)"},
    {3, {0x00, 0x21, 0x23}, "Retrokits"},
    {3, {0x00, 0x21, 0x24}, "Morningstar FX Pte Ltd"},
    {3, {0x00, 0x21, 0x25}, "Changsha Hotone Audio Co Ltd"},
    {3, {0x00, 0x21, 0x26}, "Expressive"},
    {3, {0x00, 0x21, 0x27}, "Expert Sleepers Ltd"},
    {3, {0x00, 0x21, 0x28}, "Timecode-Vision Technology"},
    {3, {0x00, 0x21, 0x29}, "Hornberg Research GbR"},
    {3, {0x00, 0x21, 0x2A}, "Sonic Potions"},
    {3, {0x00, 0x21, 0x2B}, "Audiofront"},
    {3, {0x00, 0x21, 0x2C}, "Fred's Lab"},
    {3, {0x00, 0x21, 0x2D}, "Audio Modeling"},
    {3, {0x00, 0x21, 0x2E}, "C. Bechstein Digital GmbH 00H 21H 2FH Motas Electronics Ltd"},
    {3, {0x00, 0x21, 0x30}, "MIND Music Labs"},
    {3, {0x00, 0x21, 0x31}, "Sonic Academy Ltd"},
    {3, {0x00, 0x21, 0x32}, "Bome Software"},
    {3, {0x00, 0x21, 0x33}, "AODYO SAS"},
    {3, {0x00, 0x21, 0x34}, "Pianoforce S.R.O"},
    {3, {0x00, 0x21, 0x35}, "Dreadbox P.C."},
    {3, {0x00, 0x21, 0x36}, "TouchKeys Instruments Ltd"},
    {3, {0x00, 0x21, 0x37}, "The Gigrig Ltd"},
    {3, {0x00, 0x21, 0x38}, "ALM Co"},
    {3, {0x00, 0x21, 0x39}, "CH Sound Design"},
    {1, {0x40}, "Kawai Musical Instruments MFG. CO. Ltd"},
    {1, {0x41}, "Roland Corporation"},
    {1, {0x42}, "Korg Inc."},
    {1, {0x43}, "Yamaha Corporation"},
    {1, {0x44}, "Casio Computer Co. Ltd"},
    {1, {0x46}, "Kamiya Studio Co. Ltd"},
    {1, {0x47}, "Akai Electric Co. Ltd."},
    {1, {0x48}, "Victor Company of Japan, Ltd."},
    {1, {0x4B}, "Fujitsu Limited"},
    {1, {0x4C}, "Sony Corporation"},
    {1, {0x4E}, "Teac Corporation"},
    {1, {0x50}, "Matsushita Electric Industrial Co. , Ltd"},
    {1, {0x51}, "Fostex Corporation"},
    {1, {0x52}, "Zoom Corporation"},
    {1, {0x54}, "Matsushita Communication Industrial Co., Ltd."},
    {1, {0x55}, "Suzuki Musical Instruments MFG. Co., Ltd."},
    {1, {0x56}, "Fuji Sound Corporation Ltd."},
    {1, {0x57}, "Acoustic Technical Laboratory, Inc."},
    {1, {0x59}, "Faith, Inc."},
    {1, {0x5A}, "Internet Corporation"},
    {1, {0x5C}, "Seekers Co. Ltd."},
    {1, {0x5F}, "SD Card Association"},
    {3, {0x00, 0x40, 0x00}, "Crimson Technology Inc."},
    {3, {0x00, 0x40, 0x01}, "Softbank Mobile Corp"},
    {3, {0x00, 0x40, 0x03}, "D&M Holdings Inc."},
};

const char *Sysex_Event::manufacturer() const
{
    for (const Manufacturer &m : manufacturers) {
        if (size >= m.length + 2 && !memcmp(&data[1], m.code, m.length))
            return m.name;
    }
    return _("Unknown");
}

std::string Sysex_Event::description() const
{
    return astrprintf(_("Size %zu Manufacturer \"%s\""), size, manufacturer());
}

bool load_sysex_file(const char *filename, std::vector<Sysex_Event> &event_list)
{
    FILE_u fh(fl_fopen(filename, "rb"));
    if (!fh)
        return false;

    unsigned long size;
    bool size_ok = fseek(fh.get(), 0, SEEK_END) != -1 &&
        long(size = ftell(fh.get())) != -1 &&
        fseek(fh.get(), 0, SEEK_SET) != -1;

    if (!size_ok)
        return false;

    std::unique_ptr<uint8_t[]> buffer(new uint8_t[size]);
    if (fread(buffer.get(), size, 1, fh.get()) != 1)
        return false;

    return load_sysex_data(buffer.get(), size, event_list);
}

bool load_sysex_data(const uint8_t *data, size_t size, std::vector<Sysex_Event> &event_list)
{
    // skip until the starting byte
    while (size > 0 && data[0] != 0xf0) {
        ++data;
        --size;
    }

    // load events stopping at trailing garbage
    size_t count = 0;
    while (size > 0 && data[0] == 0xf0) {
        Sysex_Event event;
        const uint8_t *end = (const uint8_t *)memchr(data, 0xf7, size);

        event.size = end ? (end - data + 1) : (size + 1);
        event.data.reset(new uint8_t[event.size]);
        event.data[0] = 0xf0;
        for (size_t i = 1; i < event.size - 1; ++i)
            event.data[i] = data[i] & 0x7f;
        event.data[event.size - 1] = 0xf7;
        event_list.push_back(std::move(event));
        ++count;

        data += event.size;
        size = end ? (size - event.size) : 0;
    }

    return count > 0;
}

static bool load_smf(const fmidi_smf_t *smf, std::vector<Sysex_Event> &event_list)
{
    fmidi_seq_u seq(fmidi_seq_new(smf));
    if (!seq)
        return false;

    size_t count = 0;
    fmidi_seq_event_t seqevent;
    while (fmidi_seq_next_event(seq.get(), &seqevent)) {
        const fmidi_event_t &event = *seqevent.event;
        if (event.type != fmidi_event_message)
            continue;

        const uint8_t *data = event.data;
        uint32_t length = event.datalen;
        if (length < 2 || data[0] != 0xf0 || data[length - 1] != 0xf7)
            continue;

        Sysex_Event syx;
        syx.data.reset(new uint8_t[length]);
        syx.size = length;
        memcpy(syx.data.get(), data, length);
        event_list.push_back(std::move(syx));
        ++count;
    }

    return count > 0;
}

bool load_smf_file(const char *filename, std::vector<Sysex_Event> &event_list)
{
    fmidi_smf_u smf(fmidi_smf_file_read(filename));
    if (!smf)
        return false;

    return load_smf(smf.get(), event_list);
}

bool load_smf_data(const uint8_t *data, size_t size, std::vector<Sysex_Event> &event_list)
{
    fmidi_smf_u smf(fmidi_smf_mem_read(data, size));
    if (!smf)
        return false;

    return load_smf(smf.get(), event_list);
}
